@@ -9,9 +9,8 @@
 #' @param dividend_yield - dividend yield
 #' @param volatility - volatility of the underlying asset
 #' @param model - the model to be chosen
-#' @param option_type in c("European", "American", "Asian", "Digital",
-#' "Binomial) - the
-#' type of option to be considered
+#' @param option_type in c("European", "American", "Asian", "Geometric Asian",
+#' "Digital", "Binomial) - the type of option to be considered
 #' @param payoff - in c("call", "put", "cash_or_nothing_call",
 #' "cash_or_nothing_put", "asset_or_nothing_call", "asset_or_nothing_put")
 #' @param greek - Greeks to be calculated in c("fair_value", "delta", "vega",
@@ -36,7 +35,17 @@ Greeks <-
            greek = c("fair_value", "delta", "vega", "theta", "rho", "gamma"),
            ...){
 
-    if (tolower(option_type) == "european" && tolower(model) == "black_scholes") {
+    if (tolower(option_type) %in% c("digital", "european") &&
+        tolower(model) == "black_scholes") {
+      if (tolower(option_type) == "digital" &&
+          tolower(payoff) %in% c("call", "put")) {
+        stop("Please choose one of the following:
+                'cash_or_nothing_call',
+                'cash_or_nothing_put',
+                'asset_or_nothing_call',
+                'asset_or_nothing_put'.")
+        return()
+      }
       return(BS_European_Greeks(payoff = payoff,
                                 greek = greek,
                                 initial_price = initial_price,
@@ -71,8 +80,7 @@ Greeks <-
     }
 
     else if (tolower(option_type) == "asian" && tolower(model) == "black_scholes" &&
-             unique(greek)[1] == "fair_value" && unique(greek)[2] == "delta" &&
-             unique(greek)[3] == "rho" && length(greek) == 3) {
+             ("fair_value" %in% greek || "delta" %in% greek || "rho" %in% greek)) {
       return(BS_Malliavin_Asian_Greeks(payoff = payoff,
                                     greek = greek,
                                     initial_price = initial_price,
@@ -94,25 +102,6 @@ Greeks <-
                                     dividend_yield = dividend_yield))
     }
 
-    else if (tolower(option_type) == "digital" && tolower(model) == "black_scholes") {
-      if (tolower(payoff) %in% c("call", "put")) {
-        stop("Please choose one of the following:
-                'cash_or_nothing_call',
-                'cash_or_nothing_put',
-                'asset_or_nothing_call',
-                'asset_or_nothing_put'.")
-        return()
-      }
-      return(BS_European_Greeks(payoff = payoff,
-                                greek = greek,
-                                initial_price = initial_price,
-                                exercise_price = exercise_price,
-                                r = r,
-                                time_to_maturity = time_to_maturity,
-                                volatility = volatility,
-                                dividend_yield = dividend_yield))
-    }
-
     else {
       stop("Wrong input or not yet implemented.")
     }
