@@ -74,7 +74,7 @@ Malliavin_Asian_Greeks <- function(
   result <-
     matrix(ncol = length(greek),
            nrow = length(vectorized_param),
-           dimnames = list(NULL, greek))
+           dimnames = list(NULL, greek)) * NA
 
   ## the payoff function ##
 
